# Posons le contexte

Prenons une convention : on va naviguer du screen __Home__ vers le screen __Magasin__, __Magasin__ est celui qui doit modifier une valeur affichée dans __Home__.

On utilise pour la navigation le module `react-navigation`, propre à React-Native.

# Vie des "screens"

Ce module de navigation comporte une navigation par pile (_stack_). Au lancement de l'application, la stack crée le screen __Home__. Au click sur un bouton de __Home__ on ouvre un nouveau screen __Magasin__ : il le crée. Dans __Magasin__ on a un bouton pour revenir en arrière avec une valeur modifiée à destination de __Home__. Lorsqu'on clique dessus, __Magasin__ est détruit.

On ne peut pas gérer ce cas avec les Lifecycles du type `componentDidUpdate()` car ces derniers sont appelés soit :

- à la construction du component
- à la modification du state du component
- à la destruction du component

On voit ici qu'il manque l'information pour __Home__ au moment du retour : il n'est ni construit, ni détruit et son state n'est pas modifié ! Il va falloir trouver une autre solution.

# Le code avant la magie

Voici la tête des différents composants avant de trouver la solution à tous nos problèmes :

## Classe `App.js`

```js
import {createStackNavigator, createAppContainer} from 'react-navigation';
import HomeScreen from './Home'
import MagasinScreen from './Magasin'

// on crée ici notre Navigateur par pile
const MainNavigator = createStackNavigator({
    Home: {screen: HomeScreen},
    Magasin: {screen: MagasinScreen}
});

// et ici le container de l'application basé sur cette pile
const App = createAppContainer(MainNavigator);

export default App;
```

## Classe `Home.js`

```js
import React, { Component } from 'react';
import { StyleSheet, Text, View, Button } from 'react-native';

export default class HomeScreen extends Component {
  static navigationOptions = {
    title: 'Welcome'
  };

  constructor(props) {
    super();
    // on définit le state d'origine
    this.state = {
      nom: 'Bhöbb',
      compte: 400
    };
  }

  _onPressButton() {
    // on se rend sur le screen Magasin en passant la
    // valeur du compte dans le "state" de la navigation
    this.props.navigation.navigate('Magasin', { 
      compte: this.state.compte
    });

  }

  render() {
    const { navigate } = this.props.navigation;

    return (
      <View style={styles.container}>
        <Text>Mes infos : </Text>
        <Text>Nom : {this.state.nom}</Text>
        <Text>$$ : {this.state.compte}</Text>
        <Button
          title='Aller au magasin'
          onPress={() => this._onPressButton()}
        />
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: '#fff',
    alignItems: 'center',
    justifyContent: 'center'
  }
});
```

## Classe `Magasin.js`

```js
import React, { Component } from 'react';
import { StyleSheet, Text, View, Button } from 'react-native';

export default class MagasinScreen extends Component {
  static navigationOptions = {
    title: 'Magasin'
  };

  constructor(props) {
    super(props);
    // on construit le state de base en récupérant le paramètre
    // passé dans le navigate de Home.js
    this.state = {
      compte: this.props.navigation.getParam('compte', 0)
    };
  }

  _onPressButton(prix) {
    // on navigue de vers Home en passant la nouvelle valeur du
    // compte
    this.props.navigation.navigate('Home', {
        compteUpdated: this.state.compte - prix;
    };
  }

  render() {
    return (
      <View style={styles.container}>
        <Text>Magasin</Text>
        <Button title='Retirer 10$$' onPress={() => this._onPressButton(10)} />
        <Button title='Retirer 20$$' onPress={() => this._onPressButton(20)} />
        <Button title='Retirer 30$$' onPress={() => this._onPressButton(30)} />
      </View>
    );
  }
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    backgroundColor: '#fff',
    alignItems: 'center',
    justifyContent: 'center'
  }
});
```
En réalité, il ne manque pas grand chose pour y arriver.

# Vivent les _callbacks_ !

Le comportement que l'on cherche à obtenir ne semble pas avoir été implémenté proprement du côté de React Navigation, j'aurais bien vu une méthode `navigate` un peu différente... de ce style là :

```js
this.props.navigation.navigate('NomDeMaPage', {des: donnees}, actionAFaireQuandLeScreenEstDetruit());
```

En gros, j'aurais bien aimé une `callback` qui se serait exécutée une fois que le composant appelé (dans notre exemple __Magasin__) serait détruit ==> ce n'est pas le cas, on va devoir le faire nous même.

# La solution

## Step one : le contenu de la callback

Dans la classe __Home__, je vais rajouter une nouvelle méthode qui aura pour mission de mettre à jour mon state quand je lui passe une donnée (pour le moment on se moque de savoir d'où elle vient, ou comment elle fait pour arriver là, c'est TRES PRECISEMENT le sujet des autres étapes !)

```js
updateCompte = (data) => {
    this.setState({
        compte: data
    })
}
```

Pour résumer ma fonction si je la lisais à voix haute :
> Ma fonction __updateCompte__ prend une donnée __data__ en paramètre et met à jour 
> le __compte__ dans le __state__ de mon composant.
> 
> et J'❤ les lambdas 😜

Et donc, quand cette fonction sera appelée, comme nous faisons un `setState()`, tout le composant va se rafraîchir ! Déjà une bonne étape.

## Step two : créer la structure pour créer la callback

Si on prend un peu de recul : par quel moyen on arrive à passer des informations à __Magasin__ depuis __Home__ ? En utilisant le `navigate()` !

Dans l'objet que l'on passe en paramètres, on est habitué à passer des valeurs :
```js
{ 
    compte: this.state.compte // ceci est une valeur
}
```

Mais la ___MAGIE___ du js, c'est... qu'une fonction est aussi une valeur !

On va donc créer une nouvelle clef/valeur pour passer notre fonction, notre objet de paramètres va ressembler à ceci :

```js
{ 
    onGoBack: this.updateCompte, // on note ici l'absence de parenthèses
    compte: this.state.compte 
}
```

Donc voici un nouveau membre de mon objet que j'appelle `onGoBack` (mais on peut l'appeler toto si on veut, ce serait une terrible idée mais ça fonctionnerait). A cette clef, j'associe la méthode `updateCompte` créé auparavant, sans les parenthèses pour dire que je veux passer la "méthode" et non pas "le résultat de la méthode une fois exécutée".

Bien, maintenant il nous reste à exécuter cette callback...

## Last step : exécution de la callback.

Ce coup ci, nous sommes du côté de notre __Magasin__. 

Je reprends, on a créé une méthode, que l'on passe en "callback" dans le `navigate()`. Donc ça veut dire que dans mon __Magasin__, si je regarde `this.props.navigation.state.params`, j'aurais ceci :

```
compte: 400,
onGoBack: ƒ (data) // ma callback !!
```

A quel moment je veux mettre à jour la donnée côté __Home__ ? Lorsque je clique sur le bouton "Retirer 10$$" (ou quand j'ai reçu la réponse d'un API, ou autre...).

On a déjà une méthode qui définit ce qui va se passer lorsqu'on va cliquer sur le bouton, on va la modifier un peu...

```js
_onPressButton(prix) {
    // on appelle notre callback !
    this.props.navigation.state.params.onGoBack(this.state.compte - prix);
    // permet de fermer le screen actuel, si on avait fait un nouveau navigate
    // vers Home, on aurait créé un "nouveau" Home !
    this.props.navigation.goBack();
}
```

VOILA ! 

Non, je vais rapidement expliquer ce qu'il se passe quand même.

1. On appuie sur le bouton
2. Ca exécute `_onPressButton()`
   1. On exécute la callback `onGoBack(valeur)` en passant en paramètre la nouvelle valeur
      1. Du côté de __Home__ on exécute la méthode associée à cette callback dans le `navigate()`, à savoir : `updateCompte(data)`. Ici `data` contient la nouvelle valeur du compte
      2. On fait un `setState()` avec la nouvelle valeur
      3. Génère un update de la vue
   2. Retour dans __Magasin__. On exécute la méthode `goBack()` du `navigation`.
3. Destruction du __Magasin__
4. Affichage de __Home__ avec les données à jour !

Ce coup ci :

VOILA ! ✨





## Références
[StackOverflow](http://stackoverflow.com/questions/52552724)
